##############################################################################
# Chapter 10                                                                  #
# Sanchez, J. Time Series for Data Scientists, CUP 2023.                        #
# Case Study 1-part I                                                         #
# Program fromEIA2Rcsv-Importing-Data.R in Base R and pre-tsibble version    #
#     With this program, data downloaded from EIA on Edison electricity       #
# demand, which is in json format, is converted to .csv format for R          #
# requires json files downloaded from EIA, to be found in this same directory #
##############################################################################

#install.packages("rjson")
library("rjson")
library("lubridate")

### Making .csv dataset from EIA JSON Files###
#########################################

## In RStudio menu, go to Session -> set working directory
#--> to source file location. And run the code below. 
#the program will read the json files downloaded from 
# EIA and convert them to a .csv version to be used in R.
# =========================================

making_dataset <- function(){
  df <- data.frame() # empty dataframe to store data 
  for(i in 1:9){  # loop over the 9 json files
    curr_file <- paste0("e",i,".json", sep="") # creating file name, always "e#.json"
    data <- fromJSON(file=curr_file)$response$data  # reading json to a list and extracting data
    date <- unlist(lapply(data, `[[`, "period"))    # extracting dates
    date <- ymd_h(date)                             # convert dates to date format
    value <- unlist(lapply(data, `[[`, "value"))    # extracting energy values megawatthours
    
    df <- rbind(df, data.frame(date=date, value=value)) #adding these new values to main df
    df <- df[order(df$date),] #sorting by date
  }
  final <- data.frame("date"=(df$date), "value"=df$value)
  final
}

edison <- making_dataset()
str(edison)
head(edison)
tail(edison)

### The above has resulted in a data set called edison.csv 
### We save the data to the hard drive. 
### 
write.csv(edison,"edison.csv")  #This is how we obtained the data 
## file edison.csv that you see in the directory. 

## check for duplicates and na
length(unique(edison$date)) == length(edison$date)
sum(is.na(edison$value))

