
##############################################################################
# Chapter 10                                                                  #
# Sanchez, J. Time Series for Data Scientists, CUP 2023.                        #
# Case Study 1-part II                                                         #
# Program Step-2-ExploreFeatures.R                      #
#     This program, reads the dataset created 
# with Step I-creatingCsv4Analysis.R #
##############################################################################

#install.packages("lubridate")
library("lubridate")

## set the working directory to Session -> set working directory
# to source file location. 

# =========================================


  edison=read.csv("edison.csv", header=T)
 str(edison)
   class(edison); head(edison)
  ## We convert character date to date variable 
   edison$date=as.POSIXct(edison$date,format="%m/%d/%Y %H:%M",tz=Sys.timezone())
   str(edison)
   head(edison)
   tail(edison)
   dim(edison)
   class(edison)
   View(edison)
   
   
   length(unique(edison$date)) == length(edison$date)
   duplicated(edison$date)
   sum(is.na(edison$value))
   sum(is.na(edison$date))
   edison$date[edison$date==NA]

### Dataset to Time Series            ###
#########################################
## Install these packages if you do not have them yet.
## install.packages("Quandl"); install.packages("dygraphs"); install.packages("xts"); 
## install.packages("forecast"); install.packages("ggplot2")
  library(Quandl)
    library(dygraphs)
    library(xts)
    library(forecast)
    library(ggplot2)

## Make the time series an xts object 
   class(edison$value)
   class(edison$date)

  my_ts <- xts(edison$value, edison$date)
  head(my_ts)

#### We look at the acf to see stationary status of raw data 
  acf(edison$value, main="ACF shows daily seasonality")

#### To inspect features of the time series, we start by aggregating 
## to some extent, finding average per day, week, month and year 

hour_ts <- period.apply(my_ts, endpoints(my_ts, "hours"), mean)
hour_ts; length(hour_ts)
day_ts <- period.apply(my_ts, endpoints(my_ts, "days"), mean)
head(day_ts); length(day_ts)
week_ts <- period.apply(my_ts, endpoints(my_ts, "weeks"), mean)
head(week_ts); length(week_ts)
month_ts <- period.apply(my_ts, endpoints(my_ts, "months"), mean)
head(month_ts); length(month_ts)
year_ts <- period.apply(my_ts, endpoints(my_ts, "years"), mean)
head(year_ts); length(year_ts)

# Time plots of the average electricity demand per year, per month, 
# per day, and per hour. 

plot.xts(year_ts, 
         ylab="Average electricity demand per year (megawatthours)",
         main="Ignoring incomplete years 2018 and 2023 \n
          annual average lowest in 2020")
plot.xts(month_ts, 
         ylab="Average electricity demand per month (megawatthours)",
         main="We notice lower monthly averages in 2019-2021")
plot.xts(day_ts, 
         ylab="Average electricity demand per day (megawatthours)",
         main="The daily averages are higher in the Summer months"
         )
plot.xts(hour_ts,
         ylab="Average electricity deman per hour (megawatthours) ",
         main="Time plot of the downloaded data, by hour"
)

# split monthly time series by type of frequency, to observe whether variability
# is affected by the frequency or higher frequencies. 

## variability of demand within hours of each day. Hard to visualize 
daily_list <- split(my_ts, f = "days")
boxplot(daily_list, 
        ylab="Demand across hours of a day (megawatthours)",
        main="Hard to visualize, as there are lots of days  \n 
          but we see summer months have more daily variability" )

# variability of demand whithin hours of each week 
weekly_list <- split(my_ts, f = "week")
boxplot(weekly_list, 
        ylab="Demand across hours of a week (megawatthours)",
        main="Behavior of demand across hours of a week   \n 
          reveals more variability and higher demand in Summer weeks" )

# variability of demand within hours of each month
monthly_list <- split(my_ts, f = "months")
boxplot(monthly_list, 
        ylab="Demand across hours of each month (megawatthours)",
        main="Behavior of demand across hours of month \n 
          only varies by month, similar across years" )

# Variability of demand within hours of each year 
yearly_list <- split(my_ts, f = "years")
boxplot(yearly_list, 
        ylab="Demand across hours of a year (megawatthours)",
        main="Behavior of demand similar across years  \n 
          ignoring the incomplete 2018 and 2023" )


#### Checking whether there are duplicates or not.


length(unique(edison$date)) == length(edison$date)
sum(is.na(edison$value))

